-- SUGOI global stuff:
-- sugoi.FailLevel() causes the level to be considered "failed", which prevents awarding emblems or setting the map cleared status. Exiting the level itself should still handled by the script itself.
-- sugoi.OLDC.pausetimer() prevents both player time and Marathon Mode time from increasing
-- sugoi.IsHubLevel() returns true if a hub spot was found, false otherwise.
-- sugoi.GetClient() returns the player that the hud is rendered for (dubbed the "client", for the best approximation of client-sided effects)
-- sugoi.ExitLevel(level, skipstats) is a replacement for G_ExitLevel. This version prevents the level from restarting twice.
-- sugoi.DoCecho(text, flags, timer) prints a cecho-like prompt, similar to the look of SRB2's unlock cechos.

OLDC.Functions = {}
local F = OLDC.Functions

function F.ShouldShowHUD()
	if OLDC.Data.voteInfo.voting and OLDC.Data.hubSpot
		return false
	end
	
	return true
end

function F.FailLevel() OLDC.Data.failedlevel = true end
function F.pausetimer() OLDC.Data.pausetimer = true end
function F.IsHubLevel() if OLDC.Data.hubSpot return true else return false end end

function F.ExitLevel(v1, v2)
	local D = OLDC.Data
	
	if D.exitingcheck return end

	G_SetCustomExitVars(v1, v2)
	G_ExitLevel()
end

function F.GetTOL(gt)
	if (gt == GT_COOP)
		if (multiplayer or netgame)
			return TOL_COOP
		else
			return TOL_SP
		end
	elseif (gt == GT_COMPETITION)
		return TOL_COMPETITION
	elseif (gt == GT_RACE)
		return TOL_RACE
	elseif (gt == GT_MATCH or gt == GT_TEAMMATCH)
		return TOL_MATCH
	elseif (gt == GT_TAG or gt == GT_HIDEANDSEEK)
		return TOL_TAG
	elseif (gt == GT_CTF)
		return TOL_CTF
	end
end

function F.NextLevel(map)
	local nextlevel = OLDC.defaultmap
	local marathonnext = mapheaderinfo[map].marathonnext
	nextlevel = mapheaderinfo[map].nextlevel

	if modeattacking then
		return nextlevel
	end
	
	if marathonmode then
		return marathonnext
	end

	if gametype == GT_COOP
		nextlevel = OLDC.hub
	elseif mapheaderinfo[nextlevel]
		-- Find next level of multiplayer gametype
		local n = nextlevel
		local t = tol(gametype)
		local type = mapheaderinfo[n].typeoflevel
		local l = 0

		while not (type & t)
			if (l > 9)
				break
			end

			n = $1+1
			if not (mapheaderinfo[n])
				n = 1 -- Go back to map01
			end

			type = mapheaderinfo[n].typeoflevel
			l = $1+1
		end

		if (type & t)
			nextlevel = n
		end
	end

	return nextlevel
end

function F.GiveEmblem(p)
	local D = OLDC.Data
	
	if not D.failedlevel
		//if mapheaderinfo[gamemap].countmap
			if splitscreen and not D.mapChecks[gamemap]
				if p and p.valid
					player.ssnumbeaten = $ + 1
				else
					for p in players.iterate
						p.ssnumbeaten = $1 + 1
					end
				end
			end

			D.mapChecks[gamemap] = true
		//end

		if not (multiplayer or modifiedgame)
		and ((players[0] and players[0].valid) and (players[0].mo and players[0].mo.valid))
			for emblem in mobjs.iterate()
				if (emblem and emblem.valid)
				and (emblem.type == MT_EMBLEM)
				and (emblem.frame == W and emblem.color == SKINCOLOR_GOLD)
					P_TeleportMove(emblem, players[0].mo.x, players[0].mo.y, players[0].mo.z)
				end
			end
		end
	end
end

function F.DEShortcut(item, enable)
	if item == "lives" and OLDC.CVars.nolives.value then
		hud.disable("lives")
		return
	end
	
	if enable
		if usingcustomhud
			customhud.enable(item)
		else
			hud.enable(item)
		end
	else
		if usingcustomhud
			customhud.disable(item)
		else
			hud.disable(item)
		end
	end
end

function F.getVoteCount(votetype)
	local D = OLDC.Data
	
	local yes = 0
	local no = 0
	local total = 0
	
	for k,v in pairs(D.playerVotes)
		if v == true
			yes = $1 + 1
			total = $1 + 1
		elseif v == false
			no = $1 + 1
			total = $1 + 1
		end
	end
	
	if (votetype == "yes" or votetype == true)
		return yes
	elseif (votetype == "no" or votetype == false)
		return no
	else
		return total
	end
end

-- v is the drawer
-- x, y are screen coordinates, fixed scale
-- text is the string to display
-- font is the patch prefix
-- alignment: -1 = right-aligned, 1 = left-aligned, 0 = centered
-- color is a skincolor
-- shadow determines whether or not to draw a pure-black copy of the text underneath, offset to the right and down
rawset(_G, "OLDC_DrawFontString", function(v, x, y, xscale, yscale, text, font, pad, spacewidth, flags, alignment, color, shadow)
	local colormap = v.getColormap(TC_DEFAULT, color or 0)
	local shadowmap = shadow and v.getColormap(TC_DEFAULT, SKINCOLOR_BLACK)
	local start
	local finish
	local dispx = x
	local dispy = y
	local items = {}
	local startitem = #items + 1
	local totalwidth = 0
	local reswidth = 0
	
	local function NewLine()
		dispx = x
		dispy = $ + (v.cachePatch(font..32).height + 1)*yscale // height of the space character + 1
		
		if alignment > 0
			return
		end
		if alignment == 0
			totalwidth = $/2
		end
		
		for j = startitem, #items
			items[j].x = $ - totalwidth
		end
		startitem = #items + 1
		totalwidth = 0
	end
	
	text = tostring(text)
	
	start = 1
	finish = text:len()
	
	for i = start, finish
		local letter = text:sub(i, i):byte()
		
		if letter == 10
			NewLine()
			continue
		elseif letter == 32
			dispx = $ + (spacewidth + 1) * xscale
			totalwidth = $ + (spacewidth + 1) * xscale
			continue
		end
		
		letter = font .. (pad and ($ < 100 and "0" or "") or "") .. $
		if not v.patchExists(letter)
			continue
		end
		
		local patch = v.cachePatch(letter)
		local width = (patch.width + 1)*xscale
		
		table.insert(items, {patch = patch, x = dispx, y = dispy})
		dispx = $ + width
		totalwidth = $ + width
	end
	
	reswidth = totalwidth
	
	NewLine()
	
	if shadow
		for i, item in ipairs(items)
			v.drawStretched(item.x + xscale, item.y + yscale, xscale, yscale, item.patch, flags, shadowmap)
		end
	end
	
	for i, item in ipairs(items)
		v.drawStretched(item.x, item.y, xscale, yscale, item.patch, flags, colormap)
	end
	
	return reswidth
end)